'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const cluster = require("cluster");
const os = require("os");
const configuration_1 = require("./configuration");
const typescript_ioc_1 = require("typescript-ioc");
const gateway_1 = require("./gateway");
const database_1 = require("./database");
const upgrade_1 = require("./utils/upgrade");
const chalk_1 = require("chalk");
class Application {
    start() {
        if (configuration_1.Configuration.instances === 1) {
            this.standalone();
        }
        else {
            this.cluster(configuration_1.Configuration.instances);
        }
    }
    standalone() {
        this.startGateway()
            .catch((err) => {
            console.error(chalk_1.default.red(`Error starting gateway: ${err.message}`));
            process.exit(-1);
        });
    }
    cluster(instances) {
        if (cluster.isMaster) {
            const n = instances < 1 ? os.cpus().length : instances;
            console.info(`Starting child processes...`);
            for (let i = 0; i < n; i++) {
                const env = { processNumber: i + 1 };
                const worker = cluster.fork(env);
                worker.process['env'] = env;
            }
            cluster.on('online', function (worker) {
                console.info(`Child process running PID: ${worker.process.pid} PROCESS_NUMBER: ${worker.process['env'].processNumber}`);
            });
            cluster.on('exit', function (worker, code, signal) {
                console.info(`PID ${worker.process.pid}  code: ${code}  signal: ${signal}`);
                const env = worker.process['env'];
                const newWorker = cluster.fork(env);
                newWorker.process['env'] = env;
            });
        }
        else {
            this.startGateway()
                .catch((err) => {
                console.error(chalk_1.default.red(`Error starting gateway: ${err.message}`));
                process.exit(-1);
            });
        }
        process.on('uncaughtException', function (err) {
            console.error(err);
        });
    }
    startGateway() {
        return new Promise((resolve, reject) => {
            const config = typescript_ioc_1.Container.get(configuration_1.Configuration);
            if (config.loaded) {
                this.runGateway().then(resolve).catch(reject);
            }
            else {
                config.on('load', () => {
                    this.runGateway().then(resolve).catch(reject);
                });
                config.on('error', error => {
                    this.ensureConfigVersionIsUpdated()
                        .then(() => config.reload())
                        .then(() => this.runGateway())
                        .then(resolve)
                        .catch(() => reject(error));
                });
            }
        });
    }
    async runGateway() {
        const gateway = typescript_ioc_1.Container.get(gateway_1.Gateway);
        const database = typescript_ioc_1.Container.get(database_1.Database);
        if (gateway.running) {
            return;
        }
        async function graceful() {
            await gateway.stopAdmin();
            await gateway.stop();
            await database.disconnect();
            process.exit(0);
        }
        // Stop graceful
        process.on('SIGTERM', graceful);
        process.on('SIGINT', graceful);
        await this.ensureConfigVersionIsUpdated();
        await gateway.start();
        await gateway.startAdmin();
        await database.registerGatewayVersion();
    }
    ensureConfigVersionIsUpdated() {
        const versions = typescript_ioc_1.Container.get(upgrade_1.VersionUpgrades);
        return versions.checkGatewayVersion();
    }
}
exports.Application = Application;
//# sourceMappingURL=application.js.map