'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const authentication_1 = require("./authentication");
const cors_1 = require("./cors");
const throttling_1 = require("./throttling");
const cache_1 = require("./cache");
const proxy_1 = require("./proxy");
const group_1 = require("./group");
const circuit_breaker_1 = require("./circuit-breaker");
const filter_1 = require("./filter");
const errors_1 = require("./errors");
const middleware_1 = require("./middleware");
const bson_1 = require("bson");
const errorHandlerSchema = Joi.object().keys({
    middleware: middleware_1.middlewareConfigValidatorSchema,
    use: Joi.string()
}).xor('middleware', 'use');
const interceptorSchema = Joi.object().keys({
    group: Joi.alternatives([Joi.array().items(Joi.string()), Joi.string()]),
    middleware: middleware_1.middlewareConfigValidatorSchema,
    use: Joi.string()
});
const interceptorsSchema = Joi.object().keys({
    request: Joi.alternatives([Joi.array().items(interceptorSchema), interceptorSchema]),
    response: Joi.alternatives([Joi.array().items(interceptorSchema), interceptorSchema])
}).min(1);
exports.apiConfigValidatorSchema = Joi.object().keys({
    authentication: Joi.alternatives([Joi.array().items(authentication_1.apiAuthenticationValidatorSchema), authentication_1.apiAuthenticationValidatorSchema]),
    cache: Joi.alternatives([Joi.array().items(cache_1.apiCacheConfigValidatorSchema), cache_1.apiCacheConfigValidatorSchema]),
    circuitBreaker: Joi.alternatives([Joi.array().items(circuit_breaker_1.apiCircuitBreakerConfigValidatorSchema), circuit_breaker_1.apiCircuitBreakerConfigValidatorSchema]),
    cors: Joi.alternatives([Joi.array().items(cors_1.apiCorsConfigSchema), cors_1.apiCorsConfigSchema]),
    description: Joi.string(),
    disableAnalytics: Joi.boolean(),
    errorHandler: errorHandlerSchema,
    filter: Joi.alternatives([Joi.array().items(filter_1.apiFilterSchema), filter_1.apiFilterSchema]),
    group: Joi.alternatives([Joi.array().items(group_1.groupValidatorSchema), group_1.groupValidatorSchema]),
    id: Joi.string(),
    interceptor: interceptorsSchema,
    name: Joi.string().min(3).required(),
    parseCookies: Joi.boolean(),
    parseReqBody: Joi.alternatives([Joi.string().valid('json', 'urlencoded', 'raw'), Joi.array().items(Joi.string().valid('json', 'urlencoded', 'raw')), Joi.boolean()]),
    path: Joi.string().regex(/^[A-Za-z\-\/0-9_\.]+$/i).required(),
    proxy: proxy_1.proxyValidatorSchema.required(),
    throttling: Joi.alternatives([Joi.array().items(throttling_1.apiThrottlingConfigValidatorSchema), throttling_1.apiThrottlingConfigValidatorSchema]),
    version: Joi.alternatives(Joi.string(), Joi.number()).required()
});
function validateApiConfig(apiConfig, disableApiIdValidation) {
    return new Promise((resolve, reject) => {
        Joi.validate(apiConfig, exports.apiConfigValidatorSchema, (err, value) => {
            if (err) {
                reject(new errors_1.ValidationError(err));
            }
            else {
                if (disableApiIdValidation) {
                    return resolve(value);
                }
                if (value.id && !bson_1.ObjectID.isValid(value.id)) {
                    return reject(new errors_1.ValidationError(`Invalid API Id ${value.id}. The Id must be a valid ObjectID. To skip this validation, configure the 'disableApiIdValidation' property on tree-gateway.yml config file.`));
                }
                return resolve(value);
            }
        });
    });
}
exports.validateApiConfig = validateApiConfig;
//# sourceMappingURL=api.js.map