import * as Joi from 'joi';
import { MiddlewareConfig } from './middleware';
/**
 * Configure support for Cors requests.
 */
export interface CorsConfig {
    /**
     * Configures the Access-Control-Allow-Origin CORS header.
     */
    origin: CorsOrigin;
    /**
     * Configures the Access-Control-Allow-Methods CORS header.
     * Expects an array (ex: ['GET', 'PUT', 'POST']).
     */
    methods?: Array<string>;
    /**
     * Configures the Access-Control-Allow-Headers CORS header.
     * Expects an array (ex: ['Content-Type', 'Authorization']).
     * If not specified, defaults to reflecting the headers specified in the request's Access-Control-Request-Headers header.
     */
    allowedHeaders?: Array<string>;
    /**
     * Configures the Access-Control-Expose-Headers CORS header.
     * Expects an array (ex: ['Content-Range', 'X-Content-Range']).
     * If not specified, no custom headers are exposed.
     */
    exposedHeaders?: Array<string>;
    /**
     * Configures the Access-Control-Allow-Credentials CORS header.
     * Set to true to pass the header, otherwise it is omitted.
     */
    credentials?: boolean;
    /**
     * Configures the Access-Control-Max-Age CORS header.
     * Set to an [human-interval](https://www.npmjs.com/package/human-interval) string to pass the header, otherwise it is omitted.
     */
    maxAge?: string;
    /**
     * Pass the CORS preflight response to the next handler.
     */
    preflightContinue?: boolean;
}
/**
 * Configure support for Cors requests.
 */
export interface ApiCorsConfig extends CorsConfig {
    /**
     * A list of groups that will use this cors cofiguration
     */
    group?: Array<string>;
    /**
     * Import a configuration from gateway config session
     */
    use?: string;
}
/**
 * Configures the Access-Control-Allow-Origin CORS header.
 */
export interface CorsOrigin {
    /**
     * If true, enable all origins to make cors requests.
     */
    enableAll?: boolean;
    /**
     * If true, disable all origins to make cors requests.
     */
    disableAll?: boolean;
    /**
     * Specify which origins are allowed.
     */
    allow?: Array<CorsOriginConfig>;
    /**
     * The name of an installed ```'cors/origin'``` middleware function, called to resolve
     * if the request should be allowed.
     */
    middleware?: MiddlewareConfig;
}
/**
 * Configures the allowed cors origins.
 */
export interface CorsOriginConfig {
    /**
     * Use regular expressions to check origins that must be allowed.
     */
    regexp?: string;
    /**
     * The origin that must be allowed.
     */
    value?: string;
}
export declare let corsConfigSchema: Joi.ObjectSchema;
export declare let apiCorsConfigSchema: Joi.ObjectSchema;
