'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const admin_1 = require("./admin");
const protocol_1 = require("./protocol");
const logger_1 = require("./logger");
const cors_1 = require("./cors");
const middleware_1 = require("./middleware");
const service_discovery_1 = require("./service-discovery");
const database_1 = require("./database");
const authentication_1 = require("./authentication");
const cache_1 = require("./cache");
const circuit_breaker_1 = require("./circuit-breaker");
const throttling_1 = require("./throttling");
const errors_1 = require("./errors");
const interceptorsSchema = Joi.object().keys({
    request: Joi.object().pattern(/\w+/, middleware_1.middlewareConfigValidatorSchema),
    response: Joi.object().pattern(/\w+/, middleware_1.middlewareConfigValidatorSchema)
});
const apiPipelineConfigSchema = Joi.object().keys({
    authentication: Joi.object().pattern(/\w+/, authentication_1.authenticationValidatorSchema),
    cache: Joi.object().pattern(/\w+/, cache_1.cacheConfigValidatorSchema),
    circuitBreaker: Joi.object().pattern(/\w+/, circuit_breaker_1.circuitBreakerConfigValidatorSchema),
    cors: Joi.object().pattern(/\w+/, cors_1.corsConfigSchema),
    errorHandler: Joi.object().pattern(/\w+/, middleware_1.middlewareConfigValidatorSchema),
    filter: Joi.object().pattern(/\w+/, middleware_1.middlewareConfigValidatorSchema),
    interceptor: interceptorsSchema,
    throttling: Joi.object().pattern(/\w+/, throttling_1.throttlingConfigValidatorSchema)
});
exports.requestAnalyticsConfigSchema = Joi.object().keys({
    enabled: Joi.boolean(),
    logger: middleware_1.middlewareConfigValidatorSchema
});
exports.gatewayConfigValidatorSchema = Joi.object().keys({
    accessLogger: logger_1.accessLoggerConfigSchema,
    admin: admin_1.adminConfigValidatorSchema,
    analytics: exports.requestAnalyticsConfigSchema,
    config: apiPipelineConfigSchema,
    cors: cors_1.corsConfigSchema,
    disableApiIdValidation: Joi.boolean(),
    disableCompression: Joi.boolean(),
    errorHandler: middleware_1.middlewareConfigValidatorSchema,
    filter: Joi.alternatives([Joi.array().items(middleware_1.middlewareConfigValidatorSchema), middleware_1.middlewareConfigValidatorSchema]),
    healthcheck: Joi.string(),
    logger: logger_1.loggerConfigSchema,
    protocol: protocol_1.protocolConfigSchema,
    serviceDiscovery: service_discovery_1.serviceDiscoveryConfigValidatorSchema,
    timeout: Joi.alternatives([Joi.string(), Joi.number().positive()]),
    underProxy: Joi.boolean()
});
exports.serverConfigValidatorSchema = Joi.object().keys({
    database: database_1.databaseSchema.required(),
    gateway: exports.gatewayConfigValidatorSchema,
    middlewarePath: Joi.string().regex(/^[a-z\.\/][a-zA-Z0-9\-_\.\/]*$/),
    rootPath: Joi.string().regex(/^[a-z\.\/][a-zA-Z0-9\-_\.\/]*$/)
});
function validateGatewayConfig(gatewayConfig) {
    return new Promise((resolve, reject) => {
        Joi.validate(gatewayConfig, exports.gatewayConfigValidatorSchema, (err, value) => {
            if (err) {
                reject(new errors_1.ValidationError(err));
            }
            else {
                return resolve(value);
            }
        });
    });
}
exports.validateGatewayConfig = validateGatewayConfig;
function validateServerConfig(config) {
    return new Promise((resolve, reject) => {
        Joi.validate(config, exports.serverConfigValidatorSchema, (err, value) => {
            if (err) {
                reject(new errors_1.ValidationError(err));
            }
            else {
                return resolve(value);
            }
        });
    });
}
exports.validateServerConfig = validateServerConfig;
//# sourceMappingURL=gateway.js.map