'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const proxy_1 = require("./proxy/proxy");
const typescript_ioc_1 = require("typescript-ioc");
const error_handler_1 = require("./error/error-handler");
const throttling_1 = require("./throttling/throttling");
const cors_1 = require("./cors/cors");
const circuit_breaker_1 = require("./circuitbreaker/circuit-breaker");
const cache_1 = require("./cache/cache");
const auth_1 = require("./authentication/auth");
const filter_1 = require("./filter/filter");
const api_1 = require("../config/api");
const configuration_1 = require("../configuration");
const logger_1 = require("../logger");
const express = require("express");
const request_1 = require("./stats/request");
const bodyParser = require("body-parser");
const cookieParser = require("cookie-parser");
const getRawBody = require("raw-body");
const cors = require("cors");
const path_1 = require("../utils/path");
let ApiPileline = class ApiPileline {
    constructor() {
        this.apiRoutes = new Map();
    }
    get apis() {
        const result = new Array();
        this.installedApis.forEach(element => {
            result.push(element);
        });
        return result;
    }
    getApiConfig(apiId) {
        return this.installedApis.get(apiId);
    }
    clearRoutes() {
        this.apiRoutes.clear();
    }
    async loadApi(api, server) {
        try {
            await api_1.validateApiConfig(api, this.config.gateway.disableApiIdValidation);
            await this.loadValidApi(api, server);
        }
        catch (err) {
            this.logger.error(`Error loading api config: ${err.message}\n${JSON.stringify(api)}`);
            throw err;
        }
    }
    circuitChanged(id, state) {
        this.apiCircuitBreaker.onStateChanged(id, state);
    }
    async loadApis(configs, server) {
        try {
            this.installedApis = new Map();
            const loaders = configs.map((config) => {
                return this.loadApi(config, server);
            });
            await Promise.all(loaders);
            this.removeOldAPIs();
        }
        catch (err) {
            this.logger.error(`Error while installing API's: ${err}`);
            throw err;
        }
    }
    buildGatewayFilters(apiRouter, filters) {
        this.apiFilter.buildGatewayFilters(apiRouter, filters);
    }
    configureCors(config) {
        const corsOptions = this.apiCors.configureCorsOptions(config);
        return cors(corsOptions);
    }
    removeOldAPIs() {
        this.apiRoutes.forEach((value, apiId) => {
            if (!this.installedApis.has(apiId)) {
                this.apiRoutes.delete(apiId);
            }
        });
    }
    loadValidApi(api, server) {
        if (this.logger.isInfoEnabled()) {
            this.logger.info(`Configuring API [${api.id}] on path: ${api.path}`);
        }
        this.installedApis.set(api.id, api);
        api.path = path_1.normalizePath(api.path);
        const apiRouter = express.Router();
        if (this.requestLogger.isRequestLogEnabled(api)) {
            this.configureLogMiddleware(apiRouter, api);
        }
        if (api.parseCookies) {
            apiRouter.use(cookieParser());
        }
        if (api.parseReqBody) {
            this.configureBodyParser(api, apiRouter);
        }
        this.apiFilter.buildApiFilters(apiRouter, api, this.config.gateway.config);
        if (api.throttling) {
            if (this.logger.isDebugEnabled()) {
                this.logger.debug('Configuring API Rate Limits');
            }
            this.apiRateLimit.throttling(apiRouter, api, this.config.gateway.config);
        }
        if (api.cors) {
            if (this.logger.isDebugEnabled()) {
                this.logger.debug('Configuring API Cors support');
            }
            this.apiCors.cors(apiRouter, api, this.config.gateway.config);
        }
        else {
            this.configureDefaultCors(apiRouter);
        }
        if (api.circuitBreaker) {
            if (this.logger.isDebugEnabled()) {
                this.logger.debug('Configuring API Circuit Breaker');
            }
            this.apiCircuitBreaker.circuitBreaker(apiRouter, api, this.config.gateway.config);
        }
        if (api.authentication) {
            if (this.logger.isDebugEnabled()) {
                this.logger.debug('Configuring API Authentication');
            }
            this.apiAuth.authentication(apiRouter, api.id, api, this.config.gateway.config);
        }
        this.apiProxy.configureProxyHeader(apiRouter, api);
        if (api.cache) {
            if (this.logger.isDebugEnabled()) {
                this.logger.debug('Configuring API Cache');
            }
            this.apiCache.cache(apiRouter, api, this.config.gateway.config);
        }
        if (this.logger.isDebugEnabled()) {
            this.logger.debug('Configuring API Proxy');
        }
        this.apiProxy.proxy(apiRouter, api, this.config.gateway.config);
        this.apiErrorHandler.handle(apiRouter, api, this.config.gateway.config);
        const initializeRouter = !this.apiRoutes.has(api.id);
        this.apiRoutes.set(api.id, apiRouter);
        if (initializeRouter) {
            server.use(api.path, (req, res, next) => {
                if (this.apiRoutes.has(api.id)) {
                    this.apiRoutes.get(api.id)(req, res, next);
                }
                else {
                    next();
                }
            });
        }
    }
    configureDefaultCors(apiRouter) {
        if (this.config.gateway.cors) {
            const corsOptions = new cors_1.ApiCors().configureCorsOptions(this.config.gateway.cors);
            apiRouter.use(cors(corsOptions));
        }
    }
    configureLogMiddleware(server, api) {
        server.use((req, res, next) => {
            const requestLog = this.requestLogger.initRequestLog(req, api);
            const end = res.end;
            const requestLogger = this.requestLogger;
            res.end = function (...args) {
                requestLog.status = res.statusCode;
                requestLog.responseTime = new Date().getTime() - requestLog.timestamp;
                requestLogger.registerOccurrence(requestLog);
                res.end = end;
                res.end.apply(res, arguments);
            };
            next();
        });
    }
    configureBodyParser(api, apiRouter) {
        const limit = api.proxy.limit || '1mb';
        let parsers;
        if (Array.isArray(api.parseReqBody)) {
            parsers = api.parseReqBody;
        }
        else {
            parsers = [`${api.parseReqBody}`];
        }
        parsers.forEach((parser) => {
            if (parser === 'json') {
                apiRouter.use(bodyParser.json({ limit: limit }));
            }
            else if (parser === 'urlencoded') {
                apiRouter.use(bodyParser.urlencoded({
                    extended: true,
                    limit: limit
                }));
            }
            else {
                apiRouter.use((req, res, next) => {
                    this.maybeParseRawBody(req, limit)
                        .then((buf) => {
                        req.body = buf;
                        next();
                    })
                        .catch((err) => next(err));
                });
            }
        });
    }
    maybeParseRawBody(req, limit) {
        if (req.body) {
            return Promise.resolve(req.body);
        }
        else {
            return getRawBody(req, {
                length: req.headers['content-length'],
                limit: limit
            });
        }
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", configuration_1.Configuration)
], ApiPileline.prototype, "config", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], ApiPileline.prototype, "logger", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", proxy_1.ApiProxy)
], ApiPileline.prototype, "apiProxy", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", error_handler_1.ApiErrorHandler)
], ApiPileline.prototype, "apiErrorHandler", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", throttling_1.ApiRateLimit)
], ApiPileline.prototype, "apiRateLimit", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", cors_1.ApiCors)
], ApiPileline.prototype, "apiCors", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", circuit_breaker_1.ApiCircuitBreaker)
], ApiPileline.prototype, "apiCircuitBreaker", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", cache_1.ApiCache)
], ApiPileline.prototype, "apiCache", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", auth_1.ApiAuth)
], ApiPileline.prototype, "apiAuth", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", filter_1.ApiFilter)
], ApiPileline.prototype, "apiFilter", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", request_1.RequestLogger)
], ApiPileline.prototype, "requestLogger", void 0);
ApiPileline = __decorate([
    typescript_ioc_1.Singleton,
    typescript_ioc_1.AutoWired
], ApiPileline);
exports.ApiPileline = ApiPileline;
//# sourceMappingURL=api.js.map