'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const errors_1 = require("../error/errors");
var State;
(function (State) {
    State[State["OPEN"] = 0] = "OPEN";
    State[State["CLOSED"] = 1] = "CLOSED";
    State[State["HALF_OPEN"] = 2] = "HALF_OPEN";
})(State = exports.State || (exports.State = {}));
class CircuitBreaker extends events_1.EventEmitter {
    constructor(options) {
        super();
        this.options = options;
        this.options.stateHandler.initialState();
    }
    isOpen() {
        return this.options.stateHandler.isOpen();
    }
    isHalfOpen() {
        return this.options.stateHandler.isHalfOpen();
    }
    isClosed() {
        return this.options.stateHandler.isClosed();
    }
    forceOpen(req) {
        if (!this.options.stateHandler.forceOpen()) {
            return;
        }
        this.emit('open', req);
    }
    forceClosed(req) {
        if (!this.options.stateHandler.forceClose()) {
            return;
        }
        this.emit('close', req);
    }
    forceHalfOpen() {
        if (!this.options.stateHandler.forceHalfOpen()) {
            return;
        }
        this.emit('halfOpen');
    }
    middleware() {
        return (req, res, next) => {
            // this.emit('request');
            if (this.isOpen() || (this.isHalfOpen() && this.options.stateHandler.halfOpenCallPending)) {
                return this.fastFail(next);
            }
            else if (this.isHalfOpen() && !this.options.stateHandler.halfOpenCallPending) {
                this.options.stateHandler.halfOpenCallPending = true;
                return this.invokeApi(req, res, next);
            }
            else {
                return this.invokeApi(req, res, next);
            }
        };
    }
    onStateChanged(state) {
        this.options.stateHandler.onStateChanged(state);
    }
    get id() {
        return this.options.id;
    }
    destroy() {
        this.removeAllListeners();
        this.options.stateHandler.destroy();
    }
    invokeApi(req, res, next) {
        let operationTimeout = false;
        const timeoutID = setTimeout(() => {
            operationTimeout = true;
            this.handleTimeout(req, res, next);
        }, this.options.timeout);
        const end = res.end;
        const self = this;
        res.end = function (...args) {
            if (!operationTimeout) {
                clearTimeout(timeoutID);
                if (res.statusCode >= 500) {
                    self.options.stateHandler.halfOpenCallPending = false;
                    self.handleFailure(req, new Error('Circuit breaker API call failure')); // TODO pegar mensagem e status do response
                }
                else {
                    self.handleSuccess(req);
                }
            }
            res.end = end;
            res.end.apply(res, arguments);
        };
        return next();
    }
    fastFail(next) {
        const err = new errors_1.ProxyError(this.options.rejectMessage, this.options.rejectStatusCode);
        next(err);
        // res.status(this.options.rejectStatusCode);
        // res.end(err.message);
        this.emit('rejected', err);
    }
    handleTimeout(req, res, next) {
        const err = new errors_1.ProxyError(this.options.timeoutMessage, this.options.timeoutStatusCode);
        this.handleFailure(req, err);
        // res.status(this.options.timeoutStatusCode);
        // res.end(err.message);
        next(err);
        // this.emit('timeout', (Date.now() - startTime));
    }
    handleSuccess(req) {
        this.forceClosed(req);
        // this.emit('success');
    }
    handleFailure(req, err) {
        this.options.stateHandler.incrementFailures()
            .then(numFailures => {
            if (this.isHalfOpen() || numFailures >= this.options.maxFailures) {
                this.forceOpen(req);
            }
        });
        // this.emit('failure', err);
    }
}
exports.CircuitBreaker = CircuitBreaker;
//# sourceMappingURL=express-circuit-breaker.js.map